@@ -4,7 +4,7 @@
 /*                                                              */
 /* Main Routine                                                 */
 /****************************************************************/
-/* bgpview.c,v 1.32 2001/07/30 08:53:49 kuniaki Exp */
+/* bgpview.c,v 1.33 2001/08/13 13:38:29 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -759,15 +759,17 @@ void usage()
 	printf("       -v          : Version\n");
 	printf("       -f filename : Config File Name\n");
 	printf("		     (Defalut : %s)\n", CONF_FILE);
-	/* printf("       -d          : Daemon(Not Supported)\n"); */
+	printf("       -d          : Daemon(Not Supported)\n");
 	printf("\n");
 }
 
 int execoption(int opcn, char *val[])
 {
 	int cnt;
+	int ret;
 
 	cnt = 1;
+	ret = 0;
 	strcpy(configfile, "");
 	while(val[cnt] != NULL) {
 		if (strcmp("-v", val[cnt]) == 0) {
@@ -776,6 +778,8 @@ int execoption(int opcn, char *val[])
 		} else if (strcmp("-f", val[cnt]) == 0) {
 			strcpy(configfile, val[cnt+1]);
 			cnt++;
+		} else if (strcmp("-d", val[cnt]) == 0) {
+			ret = 2;
 		} else {
 			usage();
 			printf("Illegal Option\n");
@@ -786,7 +790,7 @@ int execoption(int opcn, char *val[])
 	if (strlen(configfile) == 0) {
 		strcpy(configfile, CONF_FILE);
 	}
-	return(0);
+	return(ret);
 }
 
 main(int argc, char *argv[])
@@ -799,14 +803,27 @@ main(int argc, char *argv[])
 	net_ulong		event_id1;
 	int			event_id2;
 	int			pn, lf, nfd;
+	int			execret;
 
 	exec_argc = argc;
 	exec_argv = argv;
 
 	/* Process Initialize */
-	if (execoption(exec_argc, exec_argv) != 0) {
-		finish(PROC_SHUTDOWN);
-		exit(1);
+	execret = execoption(exec_argc, exec_argv);
+	switch(execret) {
+		case 0:	/* Normal Execute */
+			break;
+
+		case 2:	/* Daemon Mode */
+			if (daemon(1, 1) < 0) {
+				printf("failed to become a daemon..\n");
+				exit(1);
+			}
+			break;
+
+		default:/* fail to execute */
+			finish(PROC_SHUTDOWN);
+			exit(1);
 	}
 
 	if (init() != 0) {
