@@ -3,7 +3,7 @@
 /*                                                              */
 /* BGPView Shell Commands                                       */
 /****************************************************************/
-/* bvcommand.c,v 1.41 2001/08/02 06:29:06 kuniaki Exp */
+/* bvcommand.c,v 1.45 2001/08/15 07:15:35 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 - 2001 Internet Initiative Japan Inc.
@@ -907,7 +907,7 @@ int pr_routesum(int clno, R_LIST *rlist, net_ulong neighbor)
 }
 
 #ifdef RTHISTORY
-int show_bgproute(BVS_CMD_PARAM *prm, int dtype)
+int show_bgproute(BVS_CMD_PARAM *prm, int dtype, boolean detail)
 #endif
 #ifndef RTHISTORY
 int bvc_show_bgproute_all(BVS_CMD_PARAM *prm)
@@ -946,7 +946,7 @@ int bvc_show_bgproute_all(BVS_CMD_PARAM *prm)
 ***/
 	} else {
 #ifdef RTHISTORY
-		pr_routesum(clno, node->rinfo, 0, false, dtype);
+		pr_routesum(clno, node->rinfo, 0, detail, dtype);
 #endif
 #ifndef RTHISTORY
 		pr_routesum(clno, node->rinfo, 0);
@@ -969,17 +969,32 @@ int bvc_show_bgproute_all(BVS_CMD_PARAM *prm)
 #ifdef RTHISTORY
 int bvc_show_bgproute_all(BVS_CMD_PARAM *prm)
 {
-	return(show_bgproute(prm, RTV_ALL));
+	return(show_bgproute(prm, RTV_ALL, false));
 }
 
 int bvc_show_bgproute_active(BVS_CMD_PARAM *prm)
 {
-	return(show_bgproute(prm, RTV_ACTIVE));
+	return(show_bgproute(prm, RTV_ACTIVE, false));
 }
 
 int bvc_show_bgproute_inactive(BVS_CMD_PARAM *prm)
 {
-	return(show_bgproute(prm, RTV_INACTIVE));
+	return(show_bgproute(prm, RTV_INACTIVE, false));
+}
+
+int bvc_show_bgproute_all_detail(BVS_CMD_PARAM *prm)
+{
+	return(show_bgproute(prm, RTV_ALL, true));
+}
+
+int bvc_show_bgproute_active_detail(BVS_CMD_PARAM *prm)
+{
+	return(show_bgproute(prm, RTV_ACTIVE, true));
+}
+
+int bvc_show_bgproute_inactive_detail(BVS_CMD_PARAM *prm)
+{
+	return(show_bgproute(prm, RTV_INACTIVE, true));
 }
 #endif
 
@@ -1210,6 +1225,112 @@ int bvc_output_bgproute_inactive_detail(BVS_CMD_PARAM *prm){
 }
 #endif
 
+#ifdef RTHISTORY
+int output_bgproute_nei(BVS_CMD_PARAM *prm, boolean detail, int dtype)
+#endif
+#ifndef RTHISTORY
+int bvc_output_bgproute_all_nei(BVS_CMD_PARAM *prm)
+#endif
+{
+	int		clno;
+	static RADIX_T	**cnode;
+	static int	*upcode;
+	RADIX_T		*node;
+	char		fname[FNAMESIZE];
+	char		fn[FNAMESIZE];
+	static FILE	*fp;
+	u_long		neighbor;
+
+	if (prm == NULL) {
+		cnode = (RADIX_T **)malloc(sizeof(RADIX_T *) * bvs_maxcon);
+		bzero(cnode, sizeof(RADIX_T *) * bvs_maxcon);
+		upcode = (int *)malloc(sizeof(int) * bvs_maxcon);
+		bzero(upcode, sizeof(int) * bvs_maxcon);
+		return(0);
+	}
+
+	clno = (prm->one_str64_adrs).clno;
+	strcpy(fn, (prm->one_str64_adrs).str);
+	neighbor = (prm->one_str64_adrs).addr;
+
+	if (fn[0] == '/') {
+		strcpy(fname, fn);
+	} else {
+		strcpy(fname, spooldir);
+		strcat(fname, "/");
+		strcat(fname, fn);
+	}
+
+	if (*(cnode+clno) == NULL) {
+		if ((fp = fopen(fname, "w")) == NULL) {
+			bvs_write(clno, "Could not open target file.\r\n");
+			return(BVS_CNT_END);
+		}
+
+		bvs_write(clno, "Now Outputting Routes to '");
+		bvs_write(clno, fname);
+		bvs_write(clno, "'\r\n");
+		fprintf(fp, 
+"   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\n");
+	}
+
+	node = show_route_all(*(cnode+clno), (upcode+clno));
+	if (node == NULL) {
+		if (*(cnode+clno) == NULL) {
+			fprintf(fp, "%% No Route\n");
+		}
+	} else {
+#ifdef RTHISTORY
+		output_routesum(node->rinfo, neighbor, fp, detail, dtype);
+#endif
+#ifndef RTHISTORY
+		output_routesum(node->rinfo, neighbor, fp);
+#endif
+	}
+	if (node == NULL) {
+		if (*(cnode+clno) == NULL) {
+			*(cnode+clno) = node;
+			fclose(fp);
+			bvs_write(clno, "Done(No Route)..\r\n");
+			return(BVS_CNT_END);
+		} else {
+			*(cnode+clno) = node;
+			fclose(fp);
+			bvs_write(clno, "Done..\r\n");
+			return(BVS_CNT_FINISH);
+		}
+	} else {
+		*(cnode+clno) = node;
+		return(BVS_CNT_CONTINUE);
+	}
+}
+
+#ifdef RTHISTORY
+int bvc_output_bgproute_all_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, false, RTV_ALL));
+}
+
+int bvc_output_bgproute_active_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, false, RTV_ACTIVE));
+}
+
+int bvc_output_bgproute_inactive_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, false, RTV_INACTIVE));
+}
+
+int bvc_output_bgproute_all_detail_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, true, RTV_ALL));
+}
+
+int bvc_output_bgproute_active_detail_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, true, RTV_ACTIVE));
+}
+
+int bvc_output_bgproute_inactive_detail_nei(BVS_CMD_PARAM *prm){
+	return(output_bgproute_nei(prm, true, RTV_INACTIVE));
+}
+#endif
+
 #ifdef RTHISTORY
 int show_bgproute_neighbor(BVS_CMD_PARAM *prm, int dtype)
 #endif
@@ -1882,12 +2003,66 @@ int bvc_showcronlist(BVS_CMD_PARAM *prm)
 	return(BVS_CNT_END);
 }
 
+int nexthop_count(R_LIST *rlist, NEXTHOP_LIST **hlist, net_ulong nei)
+{
+	R_LIST		*routelist;
+	ROUTE_INFO	*routeinfo;
+	NEXTHOP_LIST	*listtmp;
+	int		actcnt;
+
+	actcnt = 0;
+	routelist = rlist;
+	while(routelist != NULL) {
+		routeinfo = routelist->route_data;
+
+		if ((nei != 0) && (routeinfo->source != nei)) {
+			routelist = routelist->next;
+			continue;
+		}
+#ifdef RTHISTORY
+		if (routeinfo->active == false) {
+			routelist = routelist->next;
+			continue;
+		}
+#endif
+		actcnt++;
+
+		if (*hlist == NULL) {
+			*hlist = (NEXTHOP_LIST *)malloc(sizeof(NEXTHOP_LIST));
+			bzero((*hlist), sizeof(NEXTHOP_LIST));
+			(*hlist)->next_hop = routeinfo->next_hop;
+			(*hlist)->count = 1;
+		} else {
+			listtmp = *hlist;
+			while(1) {
+				if (listtmp->next_hop == routeinfo->next_hop) {
+					listtmp->count++;
+					break;
+				}
+				if (listtmp->next == NULL) {
+					listtmp->next = (NEXTHOP_LIST *)malloc(
+	                                                  sizeof(NEXTHOP_LIST));
+					bzero(listtmp->next, sizeof(NEXTHOP_LIST));
+					(listtmp->next)->next_hop = 
+								routeinfo->next_hop;
+					(listtmp->next)->count = 1;
+					break;
+				}
+				listtmp = listtmp->next;
+			}
+		}
+		routelist = routelist->next;
+	}
+	return(actcnt);
+}
+
 int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
 {
 	int		clno;
 	static RADIX_T	**cnode;
 	static int	*upcode;
 	static NEXTHOP_LIST	**hoplist;
+	static int	*actives;
 	RADIX_T		*node;
 	char		prline[128];
 	int		cntr;
@@ -1905,6 +2080,8 @@ int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
 		hoplist = (NEXTHOP_LIST **)malloc(sizeof(NEXTHOP_LIST *) * 
 								bvs_maxcon);
 		bzero(hoplist, sizeof(NEXTHOP_LIST *) * bvs_maxcon);
+		actives = (int *)malloc(sizeof(int) * bvs_maxcon);
+		bzero(actives, sizeof(int) * bvs_maxcon);
 		return(0);
 	}
 
@@ -1913,6 +2090,7 @@ int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
 	if (*(cnode+clno) == NULL) {
 		bvs_write(clno, "BGP Next Hop Summarizing ...");
 		*(hoplist + clno) = 0;
+		*(actives + clno) = 0;
 	}
 
 	node = show_route_all(*(cnode+clno), (upcode+clno));
@@ -1932,6 +2110,9 @@ int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
 			sprintf(prline, "  Received Nodes    : %ld\r\n", 
 								radix_nodes());
 			bvs_write(clno, prline);
+			sprintf(prline, "  Active Routes     : %ld\r\n",
+								*(actives + clno));
+			bvs_write(clno, prline);
 
 			bvs_write(clno, "\r\nNext Hop        Routes Rate\r\n");
 			listtmp = *(hoplist+clno);
@@ -1953,40 +2134,102 @@ int bvc_show_bgpnexthop_sum(BVS_CMD_PARAM *prm)
 
 		}
 	} else {
-		cptr = node->rinfo;
-		while(cptr != NULL) {
-			routeinfo = cptr->route_data;
-			if ((*(hoplist+clno)) == NULL) {
-				*(hoplist+clno) = (NEXTHOP_LIST *)malloc(
-						  sizeof(NEXTHOP_LIST));
-				bzero(*(hoplist+clno), sizeof(NEXTHOP_LIST));
-				(*(hoplist+clno))->next_hop = 
-							routeinfo->next_hop;
-				(*(hoplist+clno))->count = 1;
-			} else {
-				listtmp = *(hoplist + clno);
-				while(1) {
-					if (listtmp->next_hop == 
-							routeinfo->next_hop) {
-						listtmp->count++;
-						break;
-					}
-					if (listtmp->next == NULL) {
-						listtmp->next = 
-							(NEXTHOP_LIST *)malloc(
-	                                                  sizeof(NEXTHOP_LIST));
-						bzero(listtmp->next, 
-							sizeof(NEXTHOP_LIST));
-						(listtmp->next)->next_hop = 
-							routeinfo->next_hop;
-						(listtmp->next)->count = 1;
-						break;
-					}
-					listtmp = listtmp->next;
-				}
-			}
-			cptr = cptr->next;
+		*(actives + clno) += nexthop_count(node->rinfo, (hoplist+clno), 0);
+	}
+	if (node == NULL) {
+		*(cnode+clno) = NULL;
+		if (*(cnode+clno) == NULL) {
+			return(BVS_CNT_END);
+		} else {
+			return(BVS_CNT_FINISH);
 		}
+	} else {
+		*(cnode+clno) = node;
+		return(BVS_CNT_CONTINUE);
+	}
+}
+
+int bvc_show_bgpnexthop_sum_neighbor(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	static RADIX_T	**cnode;
+	static int	*upcode;
+	static NEXTHOP_LIST	**hoplist;
+	static int	*actives;
+	RADIX_T		*node;
+	char		prline[128];
+	int		cntr;
+	NEXTHOP_LIST	*listtmp, *ftmp;
+	R_LIST		*cptr;
+	ROUTE_INFO	*routeinfo;
+	int		allroutes;
+	struct in_addr	inaddr;
+	net_ulong	neigh;
+
+	if (prm == NULL) {
+		cnode = (RADIX_T **)malloc(sizeof(RADIX_T *) * bvs_maxcon);
+		bzero(cnode, sizeof(RADIX_T *) * bvs_maxcon);
+		upcode = (int *)malloc(sizeof(int) * bvs_maxcon);
+		bzero(upcode, sizeof(int) * bvs_maxcon);
+		hoplist = (NEXTHOP_LIST **)malloc(sizeof(NEXTHOP_LIST *) * 
+								bvs_maxcon);
+		bzero(hoplist, sizeof(NEXTHOP_LIST *) * bvs_maxcon);
+		actives = (int *)malloc(sizeof(int) * bvs_maxcon);
+		bzero(actives, sizeof(int) * bvs_maxcon);
+		return(0);
+	}
+
+	clno = (prm->adrs).clno;
+	neigh = (prm->adrs).addr;
+
+	if (*(cnode+clno) == NULL) {
+		bvs_write(clno, "BGP Next Hop Summarizing ...");
+		*(hoplist + clno) = 0;
+		*(actives + clno) = 0;
+	}
+
+	node = show_route_all(*(cnode+clno), (upcode+clno));
+	if (node == NULL) {
+		if (*(cnode+clno) == NULL) {
+			bvs_write(clno, "\r\n%% No Route\r\n");
+		}
+		else {
+			bvs_write(clno, "\r\nNext Hop  Summary ------\r\n");
+			sprintf(prline, "  Received Prefixes : %ld\r\n",
+								radix_prefixes());
+			bvs_write(clno, prline);
+			allroutes = radix_routes();
+			sprintf(prline, "  Received Routes   : %ld\r\n",
+								 allroutes);
+			bvs_write(clno, prline);
+			sprintf(prline, "  Received Nodes    : %ld\r\n", 
+								radix_nodes());
+			bvs_write(clno, prline);
+			sprintf(prline, "  Active Routes     : %ld\r\n",
+								*(actives + clno));
+			bvs_write(clno, prline);
+
+			bvs_write(clno, "\r\nNext Hop        Routes Rate\r\n");
+			listtmp = *(hoplist+clno);
+			while(listtmp != NULL) {
+				inaddr.s_addr = listtmp->next_hop;
+				sprintf(prline, "%15s %6d %5.1f%%\r\n",
+					inet_ntoa(inaddr), listtmp->count,
+					((float)(listtmp->count)/(float)allroutes)*100);
+				bvs_write(clno, prline);
+				listtmp = listtmp->next;
+			}
+			bvs_write(clno, "\r\n");
+			listtmp = *(hoplist+clno);
+			while(listtmp != NULL) {
+				ftmp = listtmp->next;
+				free(listtmp);
+				listtmp = ftmp;
+			}
+
+		}
+	} else {
+		*(actives+clno) += nexthop_count(node->rinfo,(hoplist+clno),neigh);
 	}
 	if (node == NULL) {
 		*(cnode+clno) = NULL;
