@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell Commands Header File                           */
 /****************************************************************/
-/* bvcommand.h,v 1.24 2001/07/30 08:53:49 kuniaki Exp */
+/* bvcommand.h,v 1.27 2001/08/14 03:07:01 kuniaki Exp */
 
 /*
  * Copyright (c) 2000-2001 Internet Initiative Japan Inc.
@@ -87,6 +87,11 @@ typedef union {
 		int	clno;
 		char	str[64];
 	} one_str64;
+	struct {
+		int	clno;
+		char	str[64];
+		u_long	addr;
+	} one_str64_adrs;
 	struct {
 		int	clno;
 		char	longstr[128];
@@ -126,6 +131,7 @@ extern int bvc_set_vtyrowsize();
 #ifdef RADIX
 extern int bvc_show_bgproute_all();
 extern int bvc_output_bgproute_all();
+extern int bvc_output_bgproute_all_nei();
 extern int bvc_show_bgproute_neighbor();
 extern int bvc_showipbgproute();
 extern int bvc_show_bgproute_sum();
@@ -137,19 +143,29 @@ extern int bvc_clear_route_history();
 extern int bvc_next_rthistory_clear_set();
 extern int bvc_show_bgproute_active();
 extern int bvc_show_bgproute_inactive();
+extern int bvc_show_bgproute_all_detail();
+extern int bvc_show_bgproute_active_detail();
+extern int bvc_show_bgproute_inactive_detail();
 extern int bvc_show_bgproute_neighbor_active();
 extern int bvc_show_bgproute_neighbor_inactive();
 extern int bvc_output_bgproute_active();
+extern int bvc_output_bgproute_active_nei();
 extern int bvc_output_bgproute_inactive();
+extern int bvc_output_bgproute_inactive_nei();
 extern int bvc_output_bgproute_all_detail();
+extern int bvc_output_bgproute_all_detail_nei();
 extern int bvc_output_bgproute_active_detail();
+extern int bvc_output_bgproute_active_detail_nei();
 extern int bvc_output_bgproute_inactive_detail();
+extern int bvc_output_bgproute_inactive_detail_nei();
 #endif
 #endif
 
 extern int bvc_readcronfile();
 extern int bvc_showcronlist();
 extern int bvc_show_bgpnexthop_sum();
+extern int bvc_show_bgpnexthop_sum_neighbor();
+
 
 #ifdef DEBUG
 #ifdef RADIX
