@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell 			                        */
 /****************************************************************/
-/* bvshell.c,v 1.45 2001/08/02 05:01:49 kuniaki Exp */
+/* bvshell.c,v 1.51 2001/09/05 11:14:55 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 Internet Initiative Japan Inc.
@@ -165,13 +165,17 @@ void bvs_init_set_default()
 	/* Static Value initialize */
 	bvc_show_bgproute_all((BVS_CMD_PARAM *)0);
 	bvc_output_bgproute_all((BVS_CMD_PARAM *)0);
+	bvc_output_bgproute_all_nei((BVS_CMD_PARAM *)0);
 	bvc_show_bgproute_neighbor((BVS_CMD_PARAM *)0);
 	bvc_show_bgproute_sum((BVS_CMD_PARAM *)0);
 	bvc_showipbgproutetime_eq((BVS_CMD_PARAM *)0);
 	bvc_showipbgproutetime_le((BVS_CMD_PARAM *)0);
 	bvc_showipbgproutetime_ge((BVS_CMD_PARAM *)0);
-	bvc_clear_route_history((BVS_CMD_PARAM *)0);
 	bvc_show_bgpnexthop_sum((BVS_CMD_PARAM *)0);
+	bvc_show_bgpnexthop_sum_neighbor((BVS_CMD_PARAM *)0);
+#ifdef RTHISTORY
+	bvc_clear_route_history((BVS_CMD_PARAM *)0);
+#endif
 
 	/* Entry Commands */
 	bvs_entry_command(bvs_cmd_exit, "quit", "Logout BGPView Shell", false);
@@ -222,6 +226,9 @@ void bvs_init_set_default()
 	bvs_entry_command(bvc_output_bgproute_all,
 			"do output route filename %STR", 
 			"BGP ALL Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_all_nei,
+			"do output route filename %STR neighbor %ADR", 
+			"BGP ALL Route output to file from neighbor", true);
 	bvs_entry_command(bvc_show_bgproute_neighbor,
 			"show ip bgp neighbor %ADR routes",
 			"Show BGP Route From Neighbor", true);
@@ -252,6 +259,15 @@ void bvs_init_set_default()
 	bvs_entry_command(bvc_show_bgproute_inactive,
 			"show ip bgp inactive",
 			"Show BGP inactive routes", false);
+	bvs_entry_command(bvc_show_bgproute_all_detail,
+			"show ip bgp detail",
+			"Show BGP all routes with history", false);
+	bvs_entry_command(bvc_show_bgproute_active_detail,
+			"show ip bgp active detail",
+			"Show BGP active routes with history", false);
+	bvs_entry_command(bvc_show_bgproute_inactive_detail,
+			"show ip bgp inactive detail",
+			"Show BGP inactive routes with history", false);
 	bvs_entry_command(bvc_show_bgproute_neighbor_active,
 			"show ip bgp neighbor %ADR active routes",
 			"Show BGP Active Route From Neighbor", true);
@@ -261,18 +277,33 @@ void bvs_init_set_default()
 	bvs_entry_command(bvc_output_bgproute_active,
 			"do output route filename %STR active", 
 			"BGP ALL Active Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_active_nei,
+			"do output route filename %STR neighbor %ADR active", 
+			"BGP ALL Active Route output to file from neighbor", true);
 	bvs_entry_command(bvc_output_bgproute_inactive,
 			"do output route filename %STR inactive", 
 			"BGP ALL In-Active Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_inactive_nei,
+			"do output route filename %STR neighbor %ADR inactive", 
+			"BGP ALL In-Active Route output to file from neighbor", true);
 	bvs_entry_command(bvc_output_bgproute_all_detail,
 			"do output route filename %STR detail", 
 			"BGP ALL Detail Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_all_detail_nei,
+			"do output route filename %STR neighbor %ADR detail", 
+			"BGP ALL Detail Route output to file from neighbor", true);
 	bvs_entry_command(bvc_output_bgproute_active_detail,
 			"do output route filename %STR active detail", 
 			"BGP ALL Detail Active Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_active_detail_nei,
+			"do output route filename %STR neighbor %ADR active detail", 
+			"BGP ALL Detail Active Route output to file from neighbor", true);
 	bvs_entry_command(bvc_output_bgproute_inactive_detail,
 			"do output route filename %STR inactive detail", 
 			"BGP ALL Detail In-Active Route output to file", true);
+	bvs_entry_command(bvc_output_bgproute_inactive_detail_nei,
+			"do output route filename %STR neighbor %ADR inactive detail", 
+			"BGP ALL Detail In-Active Route output to file from neighbor", true);
 #endif /* RTHISTORY */
 #endif
 	bvs_entry_command(bvc_readcronfile,
@@ -284,6 +315,9 @@ void bvs_init_set_default()
 	bvs_entry_command(bvc_show_bgpnexthop_sum,
 			"show ip nexthop summary", 
 			"Show BGP Next Hop Summary", false);
+	bvs_entry_command(bvc_show_bgpnexthop_sum_neighbor,
+			"show ip nexthop summary neighbor %ADR", 
+			"Show BGP Next Hop Summary with neighbor", false);
 
 #ifdef DEBUG
 #ifdef RADIX
@@ -533,10 +567,10 @@ void bvs_readsocset(fd_set *fds)
 
 void *bvs_search_function(const char *cmdline, BVS_CMD_PARAM *param, boolean *more)
 {
-	char		cmd[MAXIMUM_CMD_LENGTH];
+	char		cmd[MAXIMUM_CMD_LENGTH+1];
 	char		*cmdptr;
 	BVS_CMDS	*curptr, *stack, *substack;
-	char		words[BVSC_WORDMAX][BVSC_WRCMAX];
+	char		words[BVSC_WORDMAX][BVSC_WRCMAX+1];
 	int		lv;
 	char		*prmptr;
 	struct in_addr  sin;
@@ -548,7 +582,8 @@ void *bvs_search_function(const char *cmdline, BVS_CMD_PARAM *param, boolean *mo
 	cmdptr = strtok(cmd, " ");
 	lv = 0;
 	while(cmdptr != NULL) {
-		strcpy(words[lv], cmdptr);
+		bzero(words[lv], BVSC_WRCMAX+1);
+		strncpy(words[lv], cmdptr, BVSC_WRCMAX);
 		cmdptr = strtok(NULL, " ");
 		lv++;
 	}
@@ -664,6 +699,7 @@ int bvs_exec_command(int clno)
 	int		cnt;
 	int		ret;
 	boolean		more;
+	struct in_addr	sin;
 
 	bvs_write(clno, vty_cr);
 
@@ -671,9 +707,17 @@ int bvs_exec_command(int clno)
 	if ((bvs_clients+clno)->status == BVSC_LOGIN) {
 		if (strcmp((bvs_clients+clno)->cmdline, bvs_pass) == 0) {
 			(bvs_clients+clno)->status = BVSC_INPUT_CMD;
+			sin.s_addr = htonl((bvs_clients+clno)->addr);
+			sprintf(logstr, "BVS Logged in from %s\n",
+						inet_ntoa(sin));
+			pr_log(VLOG_GLOB, LOG_ALL, logstr, ON);
 		} else {
 			bvs_write(clno, "\r\nPassword Incorrect!\r\n");
 			(bvs_clients+clno)->status = BVSC_LOGIN;
+			sin.s_addr = htonl((bvs_clients+clno)->addr);
+			sprintf(logstr, "BVS Login Password Fail : %s\n",
+						inet_ntoa(sin));
+			pr_log(VLOG_GLOB, LOG_ALL, logstr, ON);
 		}
 	} else {
 #ifdef DEBUG
@@ -683,9 +727,11 @@ int bvs_exec_command(int clno)
 		/* Search Function */
 		function = bvs_search_function((bvs_clients+clno)->cmdline,
 						 &prm, &more);
-		if (function == NULL) {
-			bvs_write(clno, "! Command Imcomplete\r\n");
-		} else {
+#ifdef DEBUG
+		printf("DEBUG: BVS: bvs_search_function OK\n");
+#endif
+
+		if (strlen((bvs_clients+clno)->cmdline) > 0) {
 			/* Entry History */
 			cnt = BVSC_MAXHISTORY-1;
 			while(cnt > 1) {
@@ -694,13 +740,26 @@ int bvs_exec_command(int clno)
 				cnt--;
 			}
 			strcpy((bvs_clients+clno)->history[1],
-						(bvs_clients+clno)->cmdline);
+			     			 (bvs_clients+clno)->cmdline);
 			if ((bvs_clients+clno)->histnum < BVSC_MAXHISTORY) {
 				(bvs_clients+clno)->histnum++;
 			}
 			(bvs_clients+clno)->curhitory = 0;
 			(bvs_clients+clno)->history[0][0] = '\0';
 
+			if (function == NULL) {
+				bvs_write(clno, "! Command Incomplete\r\n");
+			}
+		}
+#ifdef DEBUG
+		printf("DEBUG: BVS: Command Line History Record done\n");
+#endif
+
+		if (function != NULL) {
+			sprintf(logstr, "BVS: Command input : VTY%d : %s : exec\n", 
+						clno, (bvs_clients+clno)->cmdline);
+			pr_log(VLOG_GLOB, LOG_ALL, logstr, ON);
+
 			/* Execute Command */
 			prm.noparam.clno = clno;
 			/* Retcode is continue code */
@@ -726,6 +785,10 @@ int bvs_exec_command(int clno)
 				(bvs_clients+clno)->cont_code = 0;
 				break;
 			}
+		} else {
+			sprintf(logstr, "BVS: Command input : VTY%d : %s : error\n", 
+						clno, (bvs_clients+clno)->cmdline);
+			pr_log(VLOG_GLOB, LOG_ALL, logstr, ON);
 		}
 	}
 
@@ -737,7 +800,7 @@ int bvs_exec_command(int clno)
 		}
 	}
 
-	bzero((bvs_clients+clno)->cmdline, MAXIMUM_CMD_LENGTH);
+	bzero((bvs_clients+clno)->cmdline, MAXIMUM_CMD_LENGTH+1);
 	(bvs_clients+clno)->col = 0;
 }
 
@@ -799,7 +862,7 @@ int bvs_cmd_auto_complete(const char *cmdline, char *addchrs)
 	char		cmd[MAXIMUM_CMD_LENGTH];
 	char		*cmdptr;
 	BVS_CMDS	*curptr, *stack;
-	char		words[BVSC_WORDMAX][BVSC_WRCMAX];
+	char		words[BVSC_WORDMAX][BVSC_WRCMAX+1];
 	int		lv;
 	int		cptn;
 	int		nextw;
@@ -822,7 +885,8 @@ int bvs_cmd_auto_complete(const char *cmdline, char *addchrs)
 	cmdptr = strtok(cmd, " ");
 	lv = 0;
 	while(cmdptr != NULL) {
-		strcpy(words[lv], cmdptr);
+		bzero(words[lv], BVSC_WRCMAX+1);
+		strncpy(words[lv], cmdptr, BVSC_WRCMAX);
 		cmdptr = strtok(NULL, " ");
 		lv++;
 	}
@@ -910,12 +974,12 @@ int bvs_cmd_pr_help(int clno, const char *cmdline)
 	BVS_CMDS	*curptr, *stack;
 	BVS_CMDS	*ptrlist[BVSC_WORDMAX];
 	int		ptrcnt;
-	char		words[BVSC_WORDMAX][BVSC_WRCMAX];
+	char		words[BVSC_WORDMAX][BVSC_WRCMAX+1];
 	int		lv;
 	int		cptn;
 	int		nextw, prcnt;
 	char		prline[BVSC_HELPMAXC+BVSC_WRCMAX+3];
-	char		swword[BVSC_WRCMAX];
+	char		swword[BVSC_WRCMAX+1];
 	boolean		ftchk;
 
 #ifdef DEBUG
@@ -932,7 +996,8 @@ int bvs_cmd_pr_help(int clno, const char *cmdline)
 	lv = 0;
 	prcnt = 0;
 	while(cmdptr != NULL) {
-		strcpy(words[lv], cmdptr);
+		bzero(words[lv], BVSC_WRCMAX+1);
+		strncpy(words[lv], cmdptr, BVSC_WRCMAX);
 		cmdptr = strtok(NULL, " ");
 		lv++;
 	}
@@ -1150,8 +1215,9 @@ int ins_char(int clno, const char *ch, int cnt, int mode)
 #endif
 	switch(mode) {
 	case BVSE_INS:
-		if ((length + cnt) > (MAXIMUM_CMD_LENGTH-1)) {
-			cnt = cnt - (MAXIMUM_CMD_LENGTH - 1 - length);
+		if ((length + cnt) > (MAXIMUM_CMD_LENGTH)) {
+			bvs_write(clno, "\a");
+			cnt = MAXIMUM_CMD_LENGTH - length;
 			if (cnt == 0) break;
 		}
 		memmove((((bvs_clients+clno)->cmdline)+curcol+cnt),
@@ -1366,12 +1432,14 @@ int bvs_input(fd_set *fds)
 			cpnt = 0;
 			tgpnt = 0;
 
+			/*********************
 			if ((strlen(instr) + strlen((bvs_clients+cnt)->cmdline))
 							> MAXIMUM_CMD_LENGTH) {
-				instr[MAXIMUM_CMD_LENGTH - 1 -
+				instr[MAXIMUM_CMD_LENGTH -
 				    strlen((bvs_clients+cnt)->cmdline)] = '\0';
 				bvs_write(cnt, "\a");
 			}
+			************************/
 
 #ifdef DEBUG
 			printf("DEBUG: BVS: Received:");
